/*
 * Copyright (c) 2013, Intel Corporation. 
 * All rights reserved.
 * 
 * The contents of this file are released under the BSD license, you may not use this file except in compliance with the License.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of Intel Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package gov.niarl.his.privacyca;

import java.io.*;
import java.math.*;
import java.util.*;
import java.security.*;
import java.security.cert.*;
import java.security.spec.*;
import java.security.interfaces.*;

import javax.security.auth.x500.X500Principal;
import org.bouncycastle.x509.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.jce.provider.*;

/**
 * <p>The utils class contains functions that fall into two categories: those that provide 
 * utility to other classes in this package; and those that perform some cross-class 
 * functionality that is specific to this package.</p>
 * <p>This package was created for performing as a Privacy Certification Authority (Privacy CA), 
 * as specified by the Trusted Computing Group. The function <b>ProcessIdentityRequest</b> encompasses 
 * the role of a Privacy CA by taking an identity request, processing it with a CA signing key, 
 * and producing the specified data blobs containing a certificate.</p>
 * <p>The function <b>makeEkCert</b> works similarly to create a certificate for a TPM's Endorsement 
 * Key (EK certificate). This is not a defined role of a Privacy CA, however the process of 
 * creating an EK certificate is not covered in any TCG documentation, and the process of 
 * creating such a certificate is very similar to that used for processing an identity request.</p>
 * <p>The creation of an identity request is generally performed by the TSS function 
 * <b>Tspi_TPM_CollateIdentityRequest</b>. There are times, however, when it may be necessary to 
 * fabricate an identity request from its basic components. The function <b>createIdentityRequest</b> 
 * can be used to do just that. It is flexible enough to create a request in the format of any 
 * of the popular flavors of TSS.</p>
 * 
 * <p>Some of the functions provided in this class require the BouncyCastle security provider library, version 141 (for Java 1.5, this is the library named bcprov-jdk15-141.jar).</p>
 * 
 * @author schawki
 *
 */
public class TpmUtils {
	public static X509Certificate makeCert(byte[] aik, RSAPrivateKey privKey, X509Certificate caCert, int validityDays, int level) 
            throws InvalidKeySpecException, 
            NoSuchAlgorithmException, 
            CertificateEncodingException, 
            NoSuchProviderException, 
            SignatureException, 
            InvalidKeyException {
        Security.addProvider(new BouncyCastleProvider());
        PublicKey publicKey = KeyFactory.getInstance("RSA").generatePublic(new X509EncodedKeySpec(aik));
        X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();
        certGen.setSerialNumber(BigInteger.valueOf(System.currentTimeMillis()));
        certGen.setIssuerDN(caCert.getSubjectX500Principal());
        certGen.setNotBefore(new java.sql.Time(System.currentTimeMillis()));
        Calendar expiry = Calendar.getInstance();
        expiry.add(Calendar.DAY_OF_YEAR, validityDays);
        certGen.setNotAfter(expiry.getTime());
        certGen.setSubjectDN(new X500Principal(""));
        certGen.setPublicKey(publicKey);
        certGen.setSignatureAlgorithm("SHA1withRSA");
        certGen.addExtension(org.bouncycastle.asn1.x509.X509Extension.subjectAlternativeName, true, 
                new GeneralNames(new GeneralName(GeneralName.rfc822Name, new String("Identity"))));
        X509Certificate cert = certGen.generate(privKey, "BC");
        return cert;
    }
	
	public static X509Certificate generateSelfSignedX509Certificate() throws NoSuchAlgorithmException, NoSuchProviderException, CertificateEncodingException, InvalidKeyException, IllegalStateException, SignatureException {
        
	    Security.addProvider(new BouncyCastleProvider());
        int validityDays = 3652;
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA", "BC");
        keyPairGenerator.initialize(1024, new SecureRandom());
        KeyPair keyPair = keyPairGenerator.generateKeyPair();
        X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();
        X500Principal dnName = new X500Principal("CN=OATServer");
        certGen.setSerialNumber(BigInteger.valueOf(System.currentTimeMillis()));
        certGen.setSubjectDN(dnName);
        certGen.setIssuerDN(dnName); // use the same
        certGen.setNotBefore(new java.sql.Time(System.currentTimeMillis()));
        Calendar expiry = Calendar.getInstance();
        expiry.add(Calendar.DAY_OF_YEAR, validityDays);
        certGen.setNotAfter(expiry.getTime());
        certGen.setPublicKey(keyPair.getPublic());
        certGen.setSignatureAlgorithm("SHA256WithRSAEncryption");
        certGen.addExtension(org.bouncycastle.asn1.x509.X509Extension.subjectAlternativeName, true, 
                new GeneralNames(new GeneralName(GeneralName.rfc822Name, new String("Identity"))));
        X509Certificate cert = certGen.generate(keyPair.getPrivate(), "BC");
        return cert;
    }
	
	/**
	 * Retrieve a private key from a PKCS #12 store. It is expected that the P12 file will contain only one private key and one public key certificate.
	 * 
	 * @param filename The name of the P12 file.
	 * @param password The password needed to extract from the specified P12 file.
	 * @return The private key.
	 * @throws KeyStoreException Passed on from called functions.
	 * @throws IOException Passed on from called functions.
	 * @throws NoSuchAlgorithmException Passed on from called functions.
	 * @throws UnrecoverableKeyException Passed on from called functions.
	 * @throws javax.security.cert.CertificateException Passed on from called functions.
	 * @throws java.security.cert.CertificateException Passed on from called functions.
	 */
	public static RSAPrivateKey privKeyFromP12(String filename, String password) 
			throws KeyStoreException, 
			IOException, 
			NoSuchAlgorithmException, 
			UnrecoverableKeyException, 
			javax.security.cert.CertificateException, 
			java.security.cert.CertificateException {
		KeyStore caKs = KeyStore.getInstance("PKCS12");
		FileInputStream fis = new FileInputStream(filename); 
		try {
			caKs.load(fis, password.toCharArray());
		} catch (Exception e) {
			e.printStackTrace();
		}
		finally{
				fis.close();
		}
		
		//caKs.load(ConfigHelper.getResourceAsStream(filename), password.toCharArray());
		Enumeration<String> aliases = caKs.aliases();
		RSAPrivateKey privKey = null;
		while(aliases.hasMoreElements()) {
			String name = aliases.nextElement();
			privKey = (RSAPrivateKey)caKs.getKey(name, password.toCharArray());
		}
		return privKey;
	}
	/**
	 * Retrieve a public key certificate from a PKCS #12 store. It is expected that the P12 file will contain only one private key and one public key certificate.
	 * 
	 * @param filename The name of the P12 file.
	 * @param password The password needed to extract from the specified P12 file.
	 * @return The private key.
	 * @throws IOException Passed on from called functions.
	 * @throws NoSuchAlgorithmException Passed on from called functions.
	 * @throws UnrecoverableKeyException Passed on from called functions.
	 * @throws javax.security.cert.CertificateException Passed on from called functions.
	 * @throws java.security.cert.CertificateException Passed on from called functions.
	 */
	public static X509Certificate certFromP12(String filename, String password) 
			throws KeyStoreException, 
			IOException, 
			NoSuchAlgorithmException, 
			javax.security.cert.CertificateException, 
			java.security.cert.CertificateException {
		KeyStore caKs = KeyStore.getInstance("PKCS12");
		FileInputStream fis =new FileInputStream(filename);
		try {
			caKs.load(fis, password.toCharArray());
		} catch (Exception e) {
			e.printStackTrace();
		}
		finally{
				fis.close();
		}
		
		
		Enumeration<String> aliases = caKs.aliases();
		X509Certificate cert = null;
		while(aliases.hasMoreElements()) {
			String name = aliases.nextElement();
			cert = (X509Certificate)caKs.getCertificate(name);
		}
		return cert;
	}
	
	/**
     * Convert a byte array to a hexidecimal character string. The string will have no delimeter between hexidecimal duples, and has no line breaks.
     * @param b Byte array to convert
     * @return A string of hexidecimal characters
     */
    public static String byteArrayToHexString(byte[] b) {
        StringBuffer sb = new StringBuffer();
        String returnStr = "";
        for (int i = 0; i < b.length; i++) {
            String singleByte = Integer.toHexString(b[i] & 0xff);
            if (singleByte.length() != 2) singleByte = "0" + singleByte;
//          returnStr += singleByte;
            returnStr = sb.append(singleByte).toString();
        }
        return returnStr;
    }
}
